import time
from django.core.urlresolvers import resolve
from django.http import HttpRequest
from django.test import TestCase
from ddusers.views import user_list
from ddusers.models import User, Project, WebPage, Publication, DataSetType, DataSet, Country
from ddusers.views import start_page

from django.utils import translation

from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from selenium.webdriver.firefox.webdriver import WebDriver

class UserPageTest(TestCase):
    def setUp(self):
        webpage = WebPage.objects.create(title='start', main_desc_heading='Startpage', text_left_heading='Left', text_middle_heading='Middle', text_right_heading='Right')
        user1 = User.objects.create_user(username='tuser1', first_name='test1', last_name='user1', position='position1', password='pw')
        user2 = User.objects.create_user(username='tuser2', position='position2', first_name='test2', last_name='user2')
        Project.objects.create(name='project1', user=user1)
        user2.interests.add('interest1')
        user2.interests.add('interest2')
        country1 = Country.objects.create(name='country1')
        user1.countries.add(country1)
        user1.save()

    def test_get_projects(self):
        user1 = User.objects.get(username='tuser1')
        user2 = User.objects.get(username='tuser2')
        self.assertEqual(user1.get_projects()[0].name, 'project1')
        self.assertEqual(user2.get_projects().count(), 0)

    def test_get_interests(self):
        #user1 = User.objects.get(username='test_username1')
        user2 = User.objects.get(username='tuser2')
        self.assertEqual(user2.get_interests().count(), 2)
        #self.assertEqual(user2.get_projects()[0].name, raises(IndexError))

    def test_user_url_resolves_to_user_listing_view(self):
        translation.activate('en')
        found = resolve('/en/users/user/')
        #print('FUNC:', found.func)
        self.assertEqual(found.func, user_list)

    def test_user_list_lists_users(self):
        self.client.login(username='tuser1', password='pw')
        response = self.client.get('/en/users/user/')
        self.assertTemplateUsed(response, 'user_list.html')
        #self.assertContains(response, "user1", 1)#
        self.assertContains(response, "user2", 1)#
        self.assertContains(response, "user3", 0)#

    def test_user_listing_returns_correct_html(self):
        response = self.client.get('/en/')
        #self.assertTemplateUsed(response, 'index.html')
        request = HttpRequest()
        response = start_page(request)
        html = response.content.decode('utf8')
        self.assertIn('<title>Protest Data Platform - home</title>', html)

class UserProfileTests(StaticLiveServerTestCase):
    #fixtures = ['user-data.json']

    @classmethod
    def setUpClass(cls):
        super(UserProfileTests, cls).setUpClass()
         # create user
        user = User.objects.create_user(username='fx', first_name='Felix', last_name='Testuser', email=None, password='fx')
        DataSetType.objects.create(name='DataSetType Test', icon='Test Icon')
        DataSet.objects.create(user=user, title='Test DataSet')
        #Project.objects.create(user=user, name='Test Project', url='www.test-project.de')
        #publication = Publication.objects.create(author='Test Author', title='Test Publication', subtitle='Test Subtitle', year='2004', included_in='Test Included_in', url='www.test-publication.de')
        #user.publications.add(publication)

        cls.browser = WebDriver()
        cls.browser.implicitly_wait(10)

    @classmethod
    def tearDownClass(cls):
        cls.browser.quit()
        super(UserProfileTests, cls).tearDownClass()

    # user logs in
    #def test_login(self):
    #    self.browser.get('%s%s' % (self.live_server_url, '/en/login/'))
    #    username_input = self.browser.find_element_by_name("username")
    #    username_input.send_keys('fx')
    #    password_input = self.browser.find_element_by_name("password")
    #    password_input.send_keys('fx')
    #    self.browser.find_element_by_xpath('//button[@type="submit"]').click()

    def django_login(self):
        self.client.login(username='fx', password='fx') #Native django test client
        cookie = self.client.cookies['sessionid']
        self.browser.get(self.live_server_url + '/en/login/')  #browser will set cookie domain based on current page domain
        self.browser.add_cookie({'name': 'sessionid', 'value': cookie.value, 'secure': False, 'path': '/'})
        self.browser.refresh() #need to update page for logged in user
        return self.browser.get('%s%s' % (self.live_server_url, '/en/users/user/'))


    def test_user_profile_modals(self):
        """user browses to profile page and checks projects/publications modals"""
        self.django_login()

        # add project
        self.browser.get('%s%s' % (self.live_server_url, '/en/users/user/edit'))
        self.browser.find_element_by_id("project_add_button").click()
        #self.browser.find_element_by_id("id_name").click()
        #self.browser.find_element_by_id("id_name").clear()
        self.browser.find_element_by_id("id_name").send_keys("Test Project")
        self.browser.find_element_by_id("id_url").click()
        self.browser.find_element_by_id("id_url").clear()
        self.browser.find_element_by_id("id_url").send_keys("www.test-project.de")
        self.browser.find_element_by_xpath("//span[@id='projectModal-target-span']//button[.='Save changes']").click()
        self.browser.find_element_by_id("project_modal_close").click()
        bodyText = self.browser.find_element_by_xpath("//div[@id='user-projects']//h3[.='Test Project']").text
        self.assertTrue("Test Project" in bodyText)

        # check project modal
        self.browser.get('%s%s' % (self.live_server_url, '/en/users/user/edit'))
        self.browser.find_element_by_css_selector('button[class="btn btn-link btn-lg projectEdit"]').click()

        input_name = self.browser.find_element_by_id('id_name')
        self.assertEqual(
            input_name.get_attribute('value'),
            'Test Project'
        )
        input_name = self.browser.find_element_by_id('id_url')
        self.assertEqual(
            input_name.get_attribute('value'),
            'http://www.test-project.de'
        )
        self.browser.find_element_by_id('project_modal_close').click()

        # add publication
        #publication = Publication.objects.create(author='Test Author', title='Test Publication', subtitle='Test Subtitle', year='2004', included_in='Test Included_in', url='www.test-publication.de')
        self.browser.get('%s%s' % (self.live_server_url, '/en/users/user/edit'))
        self.browser.find_element_by_id("publication_add_button").click()
        self.browser.find_element_by_id("id_title").send_keys("Test Publication")
        self.browser.find_element_by_id("id_author").click()
        self.browser.find_element_by_id("id_author").clear()
        self.browser.find_element_by_id("id_author").send_keys("Test Author")
        self.browser.find_element_by_xpath("//span[@id='publicationModal-target-span']//button[.='Save changes']").click()
        self.browser.find_element_by_id("publication_modal_close").click()
        bodyText = self.browser.find_element_by_xpath("//div[@id='user-publications']//h3[.='Test Publication']").text
        self.assertTrue("Test Publication" in bodyText)



        # TODO: extend publication tests to ALL fields
        #publication = Publication.objects.create(author='Test Author', title='Test Publication', subtitle='Test Subtitle', year='2004', included_in='Test Included_in', url='www.test-publication.de')
        self.browser.get('%s%s' % (self.live_server_url, '/en/users/user/edit'))
        self.browser.find_element_by_css_selector('button[class="btn btn-link btn-lg publicationEdit"]').click()
        input_name = self.browser.find_element_by_id('id_title')
        self.assertEqual(
            input_name.get_attribute('value'),
            'Test Publication'
        )
        input_name = self.browser.find_element_by_id('id_url')
        self.assertEqual(
            input_name.get_attribute('value'),
            'www.test-publication.de'
        )

        # user browses to his dataset page
        #self.browser.get('%s%s' % (self.live_server_url, '/en/users/user/datasets'))

        # check title
        #self.assertIn('Datasets', self.browser.title)

        # check dataset listing

        # user browses to add dataset page
        #self.browser.get('%s%s' % (self.live_server_url, '/en/data/user/dataset/add'))
        #self.assertIn('Add', self.browser.title)

        # create new project
        #self.browser.find_element_by_id('project_add_button').click()
        #self.browser.find_element_by_css_selector('button[class="btn btn-link btn-lg projectAdd"]').click()

class UserProjectTestCase(TestCase):
    def setUp(self):
        user1 = User.objects.create(username='test_username1', position='test_position1')
        user2 = User.objects.create(username='test_username2', position='test_position2')
        Project.objects.create(name='test_project1', user=user1)
        user2.interests.add('test_interest1')
        user2.interests.add('test_interest2')


    def test_get_projects(self):
        user1 = User.objects.get(username='test_username1')
        user2 = User.objects.get(username='test_username2')
        self.assertEqual(user1.get_projects()[0].name, 'test_project1')
        #self.assertEqual(user2.get_projects()[0].name, raises(IndexError))

    def test_get_interests(self):
        #user1 = User.objects.get(username='test_username1')
        user2 = User.objects.get(username='test_username2')
        self.assertEqual(user2.get_interests().count(), 2)
        #self.assertEqual(user2.get_projects()[0].name, raises(IndexError))

